--[[
	##AddOn	: CooldownIcons R(evamped)
	##Subfile 	: CIREnv.lua
	##Author  : Hati ( xhatix on wowace.com )
--]]
local CIR = _G.CooldownIconsR
local icons = CIR.icons
local f
local LCI = LibStub("LibCooldownIcons") -- this is more a personal utility lib (provides some 'extra' functions or extended standard ones)
local tconcat = table.concat
local ScriptEnv = { print = print, select = select }
local funcs = { }

CIR.ScriptEnv = ScriptEnv

function CIR:ResetFuncOfIcon(icon)
    funcs[icon]=nil
end
function CIR:UnregisterObjFromEvent(obj,e)
	for event,iconT in pairs(self.events) do
		if event==e then
			tDeleteItem(iconT,obj) 
		end
	end
end
function CIR:UnregisterObjFromAllEvents(obj)
    for event,iconT in pairs(self.events) do
        for i,name in ipairs(iconT) do
            if obj==name then
                tremove(iconT,i)
            end
        end
    end
end

local noReg = {
    "CIR_GETTIME",
    "SPELL_UPDATE_COOLDOWN",
    "CIR_RANGEEVENT",
}
local specialHandlers = {
    ["UNIT_SPELLCAST"] = CIR.UNIT_SPELLCAST
}

function CIR:RegisterObjForEvent(obj,event,special)
	--Register Obj 
    if not special then
        if not self.events[event] then
            self.events[event] = { }
            if not tContains(noReg,event) then
                    self:RegisterEvent(event)
            end
        end
        if not tContains(self.events[event],obj.name) then
            tinsert(self.events[event],obj.name)
            --self.events[event][#self.events[event]+1]=obj.name
        end
    else
        if not self.events[special] then
            self.events[special]={}
            if not tContains(noReg,event) then
                self:RegisterEvent(event,specialHandlers[special])
            end
        end
        if not tContains(self.events[special],obj.name) then
            tinsert(self.events[special],obj.name)
            --self.events[event][#self.events[event]+1]=obj.name
        end
    end
	--print(LCI:inTable(obj.name,self.events[event]))
end

local function compile_script(script)
	if not funcs[f.name] then
        local func,errormsg = assert(loadstring(script))
		if errormsg then error("["..f.name.."]: "..errormsg) return end
        setfenv(func, ScriptEnv)
        func()
        funcs[f.name] = func
    else
        funcs[f.name]()
    end
end

local ENV_SpellIDFixes={
    
}

local function my_tContains(tbl,x,inQ)
    for index,value in pairs(tbl) do
        if type(value)=='table' then
            return my_tContains(value,x,index)
        elseif value==x then
            return true,(inQ or index)
        end
    end
end

local function spell(name)
	if f then
		--user input was id that require fix
        local hasFix,inQ=my_tContains(ENV_SpellIDFixes,name)
        if hasFix then
            name=inQ
            --print(inQ)
            --foreach(inQ,print)
        --user input was id
		elseif type(name) == "number" then
			name = GetSpellInfo(name)
            --checking if values for name are available - else use id again
		elseif type(name) ~= "string" then
			error(name.." is not a valid string! Might you've only forgot the quotes ( \""..name.."\" )" )
		end
		f:SetSpell(name)
	end
end
ScriptEnv.spell = spell

local function item(name)
	if f then
		if type(name) == "number" then
			name = GetItemInfo(name)
		elseif type(name) ~= "string" then
			error(name.." is not a valid string! Rember to use quoted strings ( \""..name.."\" )" )
		end
		f:SetItem(name)
	end
end
ScriptEnv.item = item

local valid_flags ={
	equip={
		[0]="ammo",
		[1]="head",
		[2]="neck",
		[3]="shoulder",
		[4]="shirt",
		[5]="chest",
		[6]="belt",
		[7]="legs",
		[8]="feet",
		[9]="wrist",
		[10]="gloves",
		[11]="finger1",
		[12]="finger2",
		[13]="trinket1",
		[14]="trinket2",
		[15]="back",
		[16]="mainhand",
		[17]="offhand",
		[18]="ranged",
		[19]="tabard",
	},
	totemtypes={
		earth=1,
		fire=2,
		water=3,
		air=4,
	},
}
CIR.ScriptEnv.valid_flags = valid_flags
local function flag(name)
	if f then
		local isFlag, id = LCI:inTable(name,valid_flags.equip)
		if not isFlag then
			error("Invalid flag, valid flags are: "..table.concat(valid_flags.equip,", "))
			return false
		else
			--f:SetItem(GetInventoryItemLink("player",id))
            f:SetInvItem(id)
			CIR:RegisterObjForEvent(f,"UNIT_INVENTORY_CHANGED")
		end
	end
end
ScriptEnv.flag = flag

--returns secondary unit ressources (required for 4.0)
local function UnitRessource(unit,sClass)
    local _, class = UnitClass(unit)
    if sClass then
        class = sClass
    end
	local sRessources = {
		["PALADIN"] = 9, --HolyPower
		--["DEATHKNIGHT"] = 5, --Runepower
        ["DRUID"] = 8, --Boomkin
        ["WARLOCK"] = 7, --Shards
        ["CAT"] = 0,
        ["BEAR"] = 0,
	}
    CIR:RegisterObjForEvent(f,"UNIT_POWER")
	return UnitPower(unit,sRessources[class] or UnitPowerType(unit))
end
ScriptEnv.UnitRessource = UnitRessource

local function PlayerCP()
    CIR:RegisterObjForEvent(f,"UNIT_COMBO_POINTS")
    return GetComboPoints("player","target")
end
ScriptEnv.PlayerCP = PlayerCP

--Conditionals
--PlayerPower - returns primary power (energy - cat, etc.)
local function PlayerPower(p)
	CIR:RegisterObjForEvent(f,"UNIT_POWER")
	if p then return UnitPower("player")/UnitPowerMax("player") else return UnitPower("player") end
end
ScriptEnv.PlayerPower = PlayerPower
--TargetPower - returns primary power of target
local function TargetPower()
	CIR:RegisterObjForEvent(f,"UNIT_TARGET")
	CIR:RegisterObjForEvent(f,"UNIT_POWER")
	return UnitPower("target")/UnitPowerMax("target")
end
ScriptEnv.TargetPower = TargetPower
--PlayerBuff - returns boolean, TODO: durationLeft
local function PlayerBuff(name,stack)
	CIR:RegisterObjForEvent(f,"UNIT_AURA")
	local n,_,_,c,_,_,d = UnitAura("player",name)
    if stack == nil then
		stack = c
	end
	return (n==name and c==stack),d,c
end
ScriptEnv.PlayerBuff = PlayerBuff
--TargetBuff
local function TargetBuff(name,stack,mine)
	CIR:RegisterObjForEvent(f,"UNIT_TARGET")
	CIR:RegisterObjForEvent(f,"UNIT_AURA")
    if not UnitExists("target") then return false,nil,nil end
    if type(stack)=='boolean' then mine=stack; stack=nil end
    if mine then mine = "PLAYER|" else mine="" end
    local n,_,_,c,_,_,d = UnitAura("target",name,nil,mine.."HELPFUL")
    if n then
        if stack==nil then
            stack = c
        end
        return (c==stack),d,c
    end
end
ScriptEnv.TargetBuff = TargetBuff
local function TargetDebuff(name,stack,mine)
	CIR:RegisterObjForEvent(f,"UNIT_TARGET")
	CIR:RegisterObjForEvent(f,"UNIT_AURA")
    if not UnitExists("target") then return false,nil,nil end
    if type(stack)=='boolean' then mine=stack; stack=nil end
    if mine then mine = "PLAYER|" else mine="" end
    local n,_,_,c,_,_,d = UnitAura("target",name,nil,mine.."HARMFUL")
    if n then
        if stack==nil then
            stack = c
        end
        return (c==stack),d,c
    end
end
ScriptEnv.TargetDebuff = TargetDebuff
local function CIR_GetTime()
    CIR:RegisterObjForEvent(f,"CIR_GETTIME")
    return GetTime()
end
ScriptEnv.GetTime = CIR_GetTime
--PlayerHP
local function PlayerHP()
    CIR:RegisterObjForEvent(f,"UNIT_HEALTH")
    return UnitHealth("player")/UnitHealthMax("player")
end
ScriptEnv.PlayerHP = PlayerHP
--TargetHP
local function TargetHP()
	CIR:RegisterObjForEvent(f,"UNIT_HEALTH")
	return UnitHealth("target")/UnitHealthMax("target")
end
ScriptEnv.TargetHP = TargetHP
--[[
	TODO:
	* get it working correctly
]]
local function SpellOnCooldown(name)
	CIR:RegisterObjForEvent(f,"SPELL_UPDATE_COOLDOWN")
	local _,duration = GetSpellCooldown(name)
	if duration then if duration>1.5 then
            return true
        end
	end
end
ScriptEnv.SpellOnCooldown = SpellOnCooldown
local function TalentGroup()
	CIR:RegisterObjForEvent(f,"PLAYER_TALENT_UPDATE")
	return GetActiveTalentGroup()
end
local function PrimaryTalentTree()
	CIR:RegisterObjForEvent(f,"PLAYER_TALENT_UPDATE")
	return GetPrimaryTalentTree()
end
local function PlayerStance()
	CIR:RegisterObjForEvent(f,"UPDATE_SHAPESHIFT_FORM")
	return GetShapeshiftForm()
end
ScriptEnv.PrimaryTalentTree = PrimaryTalentTree
ScriptEnv.TalentGroup = TalentGroup
ScriptEnv.PlayerStance = PlayerStance

local function targetIsCasting()
    CIR:RegisterObjForEvent(f,"UNIT_SPELLCAST_CHANNEL_START","UNIT_SPELLCAST")
    CIR:RegisterObjForEvent(f,"UNIT_SPELLCAST_CHANNEL_STOP","UNIT_SPELLCAST")
    CIR:RegisterObjForEvent(f,"UNIT_SPELLCAST_FAILED","UNIT_SPELLCAST")
    CIR:RegisterObjForEvent(f,"UNIT_SPELLCAST_INTERRUPTED","UNIT_SPELLCAST")
    CIR:RegisterObjForEvent(f,"UNIT_SPELLCAST_START","UNIT_SPELLCAST")
    CIR:RegisterObjForEvent(f,"UNIT_SPELLCAST_STOP","UNIT_SPELLCAST")
    CIR:RegisterObjForEvent(f,"UNIT_TARGET")
    --if UnitCastingInfo("target") then
       -- return true
    --end
    if UnitCastingInfo("target") or UnitChannelInfo("target") then return true end
end
ScriptEnv.targetIsCasting = targetIsCasting

local function inRange()
    CIR:RegisterObjForEvent(f,"CIR_RANGEEVENT")
    return f.inRange
end
ScriptEnv.inRange = inRange

local function SpellInRange(name)
    CIR:RegisterObjForEvent(f,"CIR_RANGEEVENT")
    if type(name) == 'number' then
        name=GetSpellInfo(name)
    end
    local inRange = IsSpellInRange(name,"target")
    if inRange==0 then
        return false
    else
        return true
    end
end
ScriptEnv.IsSpellInRange = SpellInRange

local function SE_CheckInteractDistance(t)
    CIR:RegisterObjForEvent(f,"CIR_RANGEEVENT")
    if CheckInteractDistance("target",t) then return true end
end
ScriptEnv.CheckInteractDistance = SE_CheckInteractDistance
--icon controls
local function GrayscaleOn()
	f:GrayscaleOn()
end
ScriptEnv.GrayscaleOn = GrayscaleOn
local function GrayscaleOff()
	f:GrayscaleOff()
end
ScriptEnv.GrayscaleOff = GrayscaleOff
local function ResetGrayscale()
	f:ResetGrayscale()
end
ScriptEnv.ResetGrayscale = ResetGrayscale
local function RangeOn()
	f:RangeColoringOn()
end
ScriptEnv.RangeOn = RangeOn
local function RangeOff()
	f:RangeColoringOff()
end
ScriptEnv.RangeOff = RangeOff
local function ResetRange()
	f:ResetRange()
end
ScriptEnv.ResetRange = ResetRange
local function SetColor(r,g,b,a)
	if r>1 or g>1 or b>1 or a>1 then
		f.texture:SetVertexColor(r/255,g/255,b/255,a/255)
		f.forcedColor = true
	else
		f.texture:SetVertexColor(r,g,b,a)
		f.forcedColor = true
	end
end
ScriptEnv.SetColor = SetColor
local function ResetColor()
	f.texture:SetVertexColor(1,1,1,1)
	f.forcedColor = false
end
ScriptEnv.ResetColor = ResetColor
local function Show()
	f:Show()
end
ScriptEnv.Show = Show
local function Hide()
	f:Hide()
end
ScriptEnv.Hide = Hide

-- returns true/false depending on player having actually a target
local function HasTarget()
	CIR:RegisterObjForEvent(f,"UNIT_TARGET")
	if UnitExists("target") then 
		return true
	else
		return false
	end
end
ScriptEnv.HasTarget = HasTarget
-- returns true/false depeding on player controlling a pet
local function HasPet()
	CIR:RegisterObjForEvent(f,"UNIT_PET")
	if UnitExists("pet") then
		return true
	else
		return false
	end
end
ScriptEnv.HasPet = HasPet
-- returns if specific pet is currently controlled by player, also allows vehicles
local function PetType(type)
	CIR:RegisterObjForEvent(f,"UNIT_PET") --LOCALPLAYER_PET_RENAMED
	--if UnitCreatureType("pet") then
	--	return true
	--else return false end
end
ScriptEnv.PetType = PetType
-- returns true/false depending on player currently controlling a earth,fire,frost,wind totem
local totemtypes={
}
local function HasTotem(totemtype)
	local hasTotemType = GetTotemInfo(valid_flags.totemtypes[totemtype])
	if hasTotemType then
		return true
	end
end
ScriptEnv.HasTotem = HasTotem

local function InBattleGround()
	CIR:RegisterObjForEvent(f,"PLAYER_ENTERING_BATTLEGROUND")
	if UnitInBattleground("player") then
		return true
	else
		return false
	end
end
ScriptEnv.InBattleground = InBattleGround

local function SE_IsUsableSpell(spell)
    CIR:RegisterObjForEvent(f,"SPELL_UPDATE_USABLE")
    return IsUsableSpell(spell)
end
ScriptEnv.IsUsableSpell = SE_IsUsableSpell

--Glow Feature
--ActionBarButtonSpellActivationAlert --BlizzardTemplate
local function CIR_Glow()
    if not f.glowFrame then
        f.glowFrame = CreateFrame("Frame", f:GetName().."_GlowFrame", UIParent, "ActionBarButtonSpellActivationAlert")
        f.glowFrame:SetParent(f)
        f.glowFrame:ClearAllPoints()
    end
    local width, height = f:GetSize()
    f.glowFrame:SetPoint("TOPLEFT", f, "TOPLEFT", -width * 0.2, height * 0.2)
    f.glowFrame:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", width * 0.2, -height*0.2)
    if not f.glowFrame.IsGlowing then
        f.glowFrame.animOut:Stop()
        f.glowFrame.animIn:Play()
        f.glowFrame.IsGlowing=true
    end
    f.glowFrame:Show()
end
local function CIR_UnGlow()
    if f.glowFrame then
        local width, height = f:GetSize()
        f.glowFrame:SetPoint("TOPLEFT", f, "TOPLEFT", -width * 0.2, height * 0.2)
        f.glowFrame:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", width * 0.2, -height*0.2)
        f.glowFrame.animIn:Stop()
        f.glowFrame.animOut:Play()
        f.glowFrame.IsGlowing=false
    end
end
ScriptEnv.Glow = CIR_Glow
ScriptEnv.UnGlow = CIR_UnGlow

local function Move(x,y)
	f:SetPoint("Center",x,y)
end
ScriptEnv.Move = Move
local function ResetMove()
	f:SetPoint("Center",CIR.db.char.icondb[f.name].iX,CIR.db.char.icondb[f.name].iY)
end
ScriptEnv.ResetMove = ResetMove
local function InCombat()
	CIR:RegisterObjForEvent(f,"PLAYER_REGEN_DISABLED")
	CIR:RegisterObjForEvent(f,"PLAYER_REGEN_ENABLED")
	return CIR.IsInCombat
end
ScriptEnv.InCombat = InCombat

local function gPulse() --there will come icon pulse in a future version
	if not tContains(CIR.pulse.iconsOnPulse,f.name) then --only support 1 pulse of that icon at a time
        tinsert(CIR.pulse.iconsOnPulse,f.name)
        local pulseFrameID = tremove(CIR.pulse.unusedPulseFrames,1)
        if not pulseFrameID then
            CIR.pulse:NewPulseFrame()
            pulseFrameID=tremove(CIR.pulse.unusedPulseFrames,1)
        end
        --foreach(pulseFrame,print)
        CIR.pulse.pulseList[pulseFrameID].icon=f.name
        CIR.pulse.pulseList[pulseFrameID].texture:SetTexture(f.texture:GetTexture())
        CIR.pulse.pulseList[pulseFrameID].anims:Play()
    end
end
ScriptEnv.gPulse = gPulse



--[[
	TODO:
	* implement some code-flags like *self which are replaced by icon 
	...
	@icon
	@text
	@texture
	
	>------<
	new commands:
	SetText(string or integer)
	SetImage(texturePath)
	DisableCooldown() - same as SetImage but you  can use item(), flag() or spell() in order to show texture
]]

function CIR:ApplyFixesToScript()
    local spellID={
        [1]=88684,
        [2]={88685,88686},
    }
    --[88685] = { "Holy Word: Sanctuary", 88686 }
    for _,st in ipairs(spellID) do
        --print("Fixing",st)
        if type(st)=='table' then
            local locale = GetSpellInfo(st[1])
            ENV_SpellIDFixes[st[1]]={}
            ENV_SpellIDFixes[st[1]][1]=locale
            ENV_SpellIDFixes[st[1]][2]=st[2]
        else
            local locale = GetSpellInfo(st)
            ENV_SpellIDFixes[st]=nil
            ENV_SpellIDFixes[st]=locale
        end
    end
    --foreach(ENV_SpellIDFixes,print)
end

function CIR:ProcessScriptForIcon(script,obj,special)
	f = nil
	f = obj
    compile_script(script)
end
