--[[
	##AddOn	: CooldownIcons R(evamped)
	##Subfile 	: icon.lua
	##Author  : Hati ( xhatix on wowace.com )
	Containing functions used by icons through ScriptEnv/Sandbox and icon class
--]]
local CIR = CooldownIconsR
local media = LibStub("LibSharedMedia-3.0")
local LBF = LibStub("LibButtonFacade", true)
local LCI = LibStub("LibCooldownIcons") -- this is more a personal utility lib (provides some 'extra' functions or extended standard ones)
CIR.icons = { }
local dftl_icon
local texture_null = "Interface\\Icons\\INV_Misc_QuestionMark"
local format,floor,ceil= format,math.floor,math.ceil

--forced range
local function RangeColoringOn(self)
	self.texture:SetVertexColor(unpack(CIR.db.char.gMisc.gRangeColor or {1,0,0,1}))
    tDeleteItem(CIR.rangeCheck,self.name)
	self.forcedrange = true
    self.inRange=false
end
local function RangeColoringOff(self)
	self.texture:SetVertexColor(1,1,1,1)
    tDeleteItem(CIR.rangeCheck,self.name)
	self.forcedrange = true
    self.inRange=true
end
local function ResetRange(self)
	local obj = self.name
	if CIR.db.char.icondb[obj].iRange then
        tinsert(CIR.rangeCheck,self.name)
		local b
		if self.type=='spell' then
			b=IsSpellInRange(self.watch,"target")
		else
			b=IsItemInRange(self.watch,"target")
		end
		if b==0 then
			self.texture:SetVertexColor(unpack(CIR.db.char.gMisc.gRangeColor or {1,0,0,1}))
            self.inRange=false
		else
			self.texture:SetVertexColor(1,1,1,1)
            self.inRange=true
		end
	else
		self.texture:SetVertexColor(1,1,1,1)
        self.inRange=nil
	end
	self.forcedrange = false
end
--forced unable
local function GrayscaleOn(self)
	self.texture:SetDesaturated(1)
	self.forcedgray = true
end
local function GrayscaleOff(self)
	self.texture:SetDesaturated(0)
	self.forcedgray = true
end
local function ResetGray(self)
	self.forcedgray = false
end
--geometric
local function GetX(self)
	return select(4,self:GetPoint())
end
local function GetY(self)
	return select(5,self:GetPoint())
end
local function GetLeft(self)
	return (self:GetX()-self:GetWidth()/2)
end
local function GetRight(self)
	return (self:GetX()+self:GetWidth()/2)
end
local function GetTop(self)
	return (self:GetY()+self:GetHeigth()/2)
end
local function GetBottom(self)
	return (self:GetY()-self:GetHeigth()/2)
end
--shortcuts
local function SetSpell(self,name)
	self.texture:SetTexture(select(3,GetSpellInfo(name)))
	self.watch = name
	self.type = 'spell'
	self:SetDisplay(false)
    --CIR:wipeCDEntry(self.name)
end
local function SetItem(self,name)
	local function GetItemID(w)
		if type(w)=='number' then
			return w
		end
		local _,itemLink = GetItemInfo(w)
        if not itemLink then
            CIR:RegisterObjForEvent(self,"GET_ITEM_INFO_RECEIVED")
            return 
        end
		return select(3,string.find(itemLink,"Hitem:(%d+)"))
	end
    --self.texture:SetTexture(select(10,GetItemInfo(name)))
	self.watch = GetItemID(name)
    self.texture:SetTexture(GetItemIcon(self.watch))
	self.type = 'item'
	self:SetDisplay(false)
end
local function SetInvItem(self,name)
    self.texture:SetTexture(GetInventoryItemTexture("player",name))
    self.watch = name
    self.type = 'flag-item'
    self:SetDisplay(false)
end
local function Clear(self)
	
end

--SetDisplay, to handle displayType
local fadingTime=1
local function SetDisplay(self,bOnCooldown)
	if self.displayType==1 then
        self:SetAlpha(1)
	else
		local alpha=self:GetAlpha()
		if self.displayType == 2 then -- OnCooldown only, fading
			if bOnCooldown and alpha~=1 then
				UIFrameFadeIn(self,(1-alpha)*fadingTime,alpha,1)
			elseif not bOnCooldown and alpha~=0 then
				UIFrameFadeIn(self,alpha*fadingTime,alpha,0)
			end
		elseif self.displayType == 3 then -- if Ready
			if bOnCooldown and alpha~=0 then
				UIFrameFadeIn(self,alpha*fadingTime,alpha,0)
			elseif not bOnCooldown and alpha~=1 then
				UIFrameFadeIn(self,(1-alpha)*fadingTime,alpha,1)
			end
		end
	end
end
--SetTime, to handle timeFormat
local function SetTime(self,start,duration)
	if self.displayType ~= 3 and self:IsVisible() then
		local now = GetTime()
		racc = CIR.db.char.gIcon.gAccuracy or 1
		local t = (duration-(now-start))
        if t<0 then self.text:SetText("") return end
		if self.timeFormat == 1 then --Seconds.Miliseconds
            t = format("%."..racc.."f",t)
		elseif self.timeFormat== 2 then --Minutes:Seconds
			local t_m = floor(t/60)
			t = format("%d:%d",t_m , floor(t%60))
		elseif self.timeFormat == 3 then --Minutes; x(1min = 0:01 - 1:00?) or (2min = 2:00 - 2:59)
			if t<60 then
			else
				t = ceil(t/60)
			end
		elseif self.timeFormat == 4 then --Seconds, if Seconds<tMin then dt =1
			if racc~=0 then
				t = format("%."..(LCI:tonumber(self.timeFormat4>=t)+(racc-1)).."f",t)
			else
				t = format("%."..racc.."f",t)
			end
		elseif self.timeFormat == 5 then
			t = ""
		end
		--self.text:SetText(math.min(t,self.text:GetText()))
		self.text:SetText(t)
	end
end
--range
CIR.rangeCheck = { }


function CIR:CreateIcon(name)
	local db = CIR.db.char.icondb
	CIR.icons[name] = { }
	local icon = CIR.icons[name]
	--... button type to support LBF ...--
	icon = CreateFrame("Button", "CIR_"..name, UIParent)
	icon.name = name
	icon:SetPoint("Center",db[name].iX,db[name].iY)
	icon:SetWidth(db[name].iIconSize)
	icon:SetHeight(db[name].iIconSize)
	icon:SetMovable(false)
	icon:EnableMouse(false)
	icon:SetScript("OnMouseDown",nil)
	icon:SetScript("OnMouseUp",nil)
	--texture
	icon.texture = icon:CreateTexture(icon:GetName().."_Texture")
	icon.texture:SetAllPoints(icon)
	icon.texture:SetTexture(texture_null)
	--text
	icon.text = icon:CreateFontString(icon:GetName().."_Text","ARTWORK")
	icon.text:SetPoint("CENTER",icon,"CENTER",db[name].iFontOffsetX,db[name].iFontOffsetY)
	icon.text:SetFont(media:Fetch("font",media:List('font')[db[name].iFontStyle] or "Fonts\\FRIZQT__.TTF"),db[name].iFontSize,"OUTLINE")
	icon.text:SetTextColor(unpack(db[name].iFontColor or {1.0,1.0,1.0,1.0}))
	
	--some variables
	icon.displayType = db[name].iDisplayType or 1
	icon.timeFormat = db[name].iTimeFormat or 1
	icon.timeFormat4 = db[name].iTimeFormat4 or 10
	--add functions
	icon.RangeColoringOn = RangeColoringOn
	icon.RangeColoringOff = RangeColoringOff
	icon.ResetRange = ResetRange
	icon.GrayscaleOn = GrayscaleOn
	icon.GrayscaleOff = GrayscaleOff
	icon.ResetGray = ResetGray
	icon.GetX = GetX
	icon.GetY = GetY
	icon.GetLeft = GetLeft
	icon.GetRight = GetRight
	icon.GetTop = GetTop
	icon.GetBottom = GetBottom
	icon.SetTime = SetTime
	icon.SetDisplay = SetDisplay
	icon.SetSpell = SetSpell
	icon.SetItem = SetItem
    icon.SetInvItem = SetInvItem
	
	if LBF then
			LBF:Group('CooldownIcons R'):AddButton( icon, {Icon=icon.texture})
	end
	local mZoom = LCI:tonumber(db[name].iZoom) *.08
	icon.texture:SetTexCoord(0.0+mZoom, 1.0-mZoom, 0.0+mZoom, 1.0-mZoom)
	
	icon.active = db[name].iEnabled or true
	CIR.icons[name] = icon
	if db[name].iRange then
		tinsert(CIR.rangeCheck,name)
	end
end







